from ... import Time, Tools, Json, Lg, String

import typing 

class BrowserTweet():
    def __init__(self) -> None:
        self.User:str = None # Screen Name 
        self.Time:int = None 
        self.Text:int = None 
        self.URL:str = None 
    
    def __str__(self) -> str:
        time = Time.Strftime(self.Time)
        return f"BrowserTweet(User={self.User} Time={time} Text={self.Text})"

    def __repr__(self) -> str:
        return self.__str__()

class Browser():
    def __init__(self, cookie:str) -> None:
        self.cookie = cookie

    def twitterSearchResultParser(self, j:dict) -> list[dict]:
        if 'globalObjects' not in j:
            return []
        
        if 'tweets' not in j['globalObjects'] or 'users' not in j['globalObjects']:
            return []

        tweets = j['globalObjects']['tweets']
        users = j['globalObjects']['users']

        res = []
        for tidx in tweets:
            tweet = tweets[tidx]

            # 采集数据
            time = tweet['created_at']
            text = tweet['full_text']

            uid = tweet['user_id_str']

            user = None 
            for uidx in users:
                if uid == users[uidx]['id_str']:
                    user = users[uidx]['screen_name']
                    break 
    
            # 处理数据
            time = Time.Strptime(time)

            # 存储
            res.append({
                "user": user,
                "time": time,
                "text": text,
                "url": f"https://twitter.com/{user}/status/{tidx}"
            })
        
        return res 

    def fetchTweet(self, se:Tools.Selenium.ChromeWire) -> list[dict]:
        resp = []
        for flow in se.Flows():
            # if not Tools.URL(flow.Request.URL).Parse().Host in ['api.twitter.com']:
            #     continue  
            # if not Tools.URL(flow.Request.URL).Parse().Path.endswith("adaptive.json"):
            #     continue 

            path = Tools.URL(flow.Request.URL).Parse().Path

            # if path in ['/i/api/2/guide.json']:
            #     continue

            if path != "/i/api/2/search/adaptive.json":
                continue

            try:
                content = Json.Loads(flow.Response.Body)
            except Exception as e:
                # Lg.Trace("错误载入json:", flow.Request.URL)
                # ipdb.set_trace()
                continue
            
            # Lg.Trace("载入json成功:", flow.Request.URL)
            for i in self.twitterSearchResultParser(content) :
                resp.append(i)
                Lg.Trace(f"有推文的url: {path} \n {i}")
        
        return resp 

    def Search(self, query:str, since:int=None, until:int=None, stype:str="top") -> typing.Iterator[BrowserTweet]:
        """
        This function searches for tweets on Twitter based on a query, time range, and search type, and
        returns an iterator of the resulting tweets.
        在stype为top的时候可能返回重复的推文
        
        :param query: The search query to be used for searching tweets
        :type query: str
        :param since: The "since" parameter is not used in the code provided. It is not clear what it is
        intended to represent without further context
        :type since: int
        :param until: The "until" parameter is used to specify the latest date (in Unix timestamp
        format) for the tweets to be searched. Only tweets posted before this date will be included in
        the search results
        :type until: int
        :param stype: The type of search results to be returned. 可以是top或者live
        :type stype: str (optional)
        """

        if since != None and type(since) not in [int, float]:
            raise Exception("since需要为数字")
        
        if until != None and type(until) not in [int, float]:
            raise Exception("until需要为数字")

        baseurl = 'https://twitter.com/search?q=%s&src=recent_search_click&f=' + stype

        if until != None:
            lastestTime = until 
        else:
            lastestTime = Time.Now()

        with Tools.Selenium.ChromeWire(randomUA=False) as se:
            se.Get("https://twitter.com")
            se.SetCookie(self.cookie)

            while True:
                utildate = Time.Strftime(lastestTime, "%Y-%m-%d")
                q = query + f" until:{utildate}"

                Lg.Info("Search:", q)
                u = baseurl % String(q).URLEncode()

                Lg.Trace("URL:", u)
                se.Get(u)

                Time.Sleep(5, bar=True, title="等待页面加载")

                empty = 0
                tweetscount = 0
                while True:
                    tweets = self.fetchTweet(se)

                    if len(tweets) == 0:
                        if f'No results for "{q}"'.lower() in se.PageSource().lower():
                            Lg.Trace("无搜索结果")
                            return 
                        
                        empty += 1
                        Lg.Trace("empty result")
                        if empty > 5:
                            Lg.Trace("break")
                            break 
                    else:
                        empty = 0

                    for i in tweets:
                        bt = BrowserTweet()
                        bt.User = i['user']
                        bt.Time = i['time']
                        bt.Text = i['text']
                        bt.URL = i['url']

                        Lg.Trace(bt)
                        
                        lastestTime = bt.Time
                        try:
                            if since != None and bt.Time < since:
                                Lg.Trace("到达since指定的日期:", Time.Strftime(since))
                                return 
                        except Exception as e:
                            Lg.Trace("since:", since)
                            Lg.Trace("bt.Time:", bt.Time)
                            Lg.Warn(e)
                        
                        tweetscount += 1
                        yield bt

                    Lg.Trace("Scrool down")
                    se.ScrollDown(6000)
                    Time.Sleep(2)
                    se.ScrollDown(6000)
                    Time.Sleep(2)

                if tweetscount == 0:
                    return 

if __name__ == "__main__":
    b = Browser(twittercookie)
    key = "RainUnlocks"
    # key = "coinsbee"
    for t in b.Search(key, Time.Now() - 86400 * 10):
        Lg.Trace(t)